/**
 * EasyBeans
 * Copyright (C) 2006-2007 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ArchiveManager.java 2045 2007-11-19 10:41:15Z fornacif $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.ee.deploy.impl.archive;

import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.util.HashMap;
import java.util.Map;

import org.ow2.util.ee.deploy.api.archive.ArchiveFactory;
import org.ow2.util.ee.deploy.api.archive.IArchive;
import org.ow2.util.ee.deploy.api.archive.IArchiveManager;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

/**
 * Creates an archive for the given type.
 * @author Florent Benoit
 */
public final class ArchiveManager implements IArchiveManager {

    /**
     * Logger.
     */
    private static Log logger = LogFactory.getLog(ArchiveManager.class);

    /**
     * Unique instance of this class.
     */
    private static ArchiveManager archiveManager;

    /**
     * Mapping between class and the associated factories.<br />
     * ie : File <--> DefaultArchiveFactory.
     */
    private Map<Class<?>, ArchiveFactory<?>> factories = new HashMap<Class<?>, ArchiveFactory<?>>();

    /**
     * Private constructor as only one instance is built.
     */
    private ArchiveManager() {
        addFactory(new DefaultArchiveFactory());
    }

    /**
     * Gets the unique instance of this class.
     * @return the unique instance.
     */
    public static ArchiveManager getInstance() {
        if (archiveManager == null) {
            archiveManager = new ArchiveManager();
        }
        return archiveManager;
    }

    /**
     * Adds the given factory on this manager. It will look the generic info
     * used by this class to find the class object managed by this factory.
     * @param factory the factory to add.
     */
    public void addFactory(final ArchiveFactory<?> factory) {

        Class<?> factoryClass = factory.getClass();
        Class<?> argumentClass = getSupportedType(factoryClass);

        logger.debug("Adding factory '" + factoryClass.getName() + "' for class '" + argumentClass + "'.");

        // Add factory
        factories.put(argumentClass, factory);

    }

    /**
     * Determines the type supported by the given factory Class object.
     * It will look the generic info used by this class to find the
     * class object managed by this factory.
     * @param factoryClass EZBArchiveFactory sub Class.
     * @return Returns the type supported by the given factory Class object.
     */
    private Class<?> getSupportedType(final Class<?> factoryClass) {

        // Get generics of this class
        Type[] generics = factoryClass.getGenericInterfaces();

        // size = 1
        if (generics.length != 1) {
            throw new IllegalArgumentException("Invalid class, supports only class with a single generic");
        }

        // parametrized type ?
        Type factoryClassType = generics[0];
        ParameterizedType parameterType = null;

        if (factoryClassType instanceof ParameterizedType) {
            parameterType = (ParameterizedType) factoryClassType;
        } else {
            throw new IllegalArgumentException("Object '" + factoryClassType + "' is not a ParameterizedType");
        }

        Type[] arguments = parameterType.getActualTypeArguments();
        if (arguments.length != 1) {
            throw new IllegalArgumentException("Invalid class, supports only class with a single generic");
        }
        return (Class<?>) arguments[0];
    }

    /**
     * Removes the given factory on this manager. It will look the generic info
     * used by this class to find the class object managed by this factory.
     * @param factory the factory to remove.
     */
    public void removeFactory(final ArchiveFactory<?> factory) {

        Class<?> factoryClass = factory.getClass();
        Class<?> argumentClass = getSupportedType(factoryClass);

        if (factories.containsKey(argumentClass)) {
            logger.debug("Removing factory '" + factoryClass.getName() + "' for class '" + argumentClass + "'.");

            // Remove factory
            factories.remove(argumentClass);
        } else {
            logger.debug("Factory '" + factoryClass.getName() + "' was not registered in the ArchiveManager.");
        }
    }

    /**
     * Creates an EZBArchive implementation object for the given object.
     * @param o object to wrap into an EZBArchive.
     * @return the created archive.
     */
    @SuppressWarnings("unchecked")
    public IArchive getArchive(final Object o) {
        // try to see if there is a matching factory
        Class objectClass = o.getClass();

        // Factory ?
        ArchiveFactory factory = factories.get(objectClass);

        if (factory == null) {

            // Iterates over the registered classes and find the assignable class
            for (Class cls : factories.keySet()) {
                if (cls.isAssignableFrom(objectClass) && factory == null) {
                    // we found a match
                    factory = factories.get(cls);
                }
            }

            // if after all, factory is still null, raise an Exception
            if (factory == null) {
                throw new IllegalArgumentException("No factory found for the type '" + objectClass + "'.");
            }
        }

        return factory.create(o);
    }

}
